/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "chost.hpp"
#include "syspaths.hpp"
#include "run_cmd.hpp"
#include "gentoofunctions.hpp"
#include "replace_bash_vars.hpp"
#include <stdexcept>
#include <cassert>

namespace duck {
	namespace {
		std::string try_real_hard_to_find_CHOST() {
			assert(false);
			throw std::runtime_error("try_real_hard_to_find_CHOST() not implemented");
	//#
	//# First we read make.conf
	//#

	//local varname=${1:-CHOST}
	//local conf=${EROOT}/etc/portage/make.conf
	//if [[ ! -e ${conf} && -e ${EROOT}/etc/make.conf ]] ; then
	//	conf=${EROOT}/etc/make.conf
	//fi
	//local ret=$(source "${conf}" 2>/dev/null ; echo ${!varname})
	//if [[ -z ${ret} ]] ; then
	//	# newer portage supports spaces between the var and =
	//	# CHOST     =  "this-is-retarded"
	//	ret=$(eval $(
	//		${SED} -n \
	//			-e 's:[[:space:]]::g' \
	//			-e "/^${varname}=/p" \
	//			"${conf}"
	//		) ; echo ${!varname}
	//	)
	//fi

	//if [[ -n ${ret} ]] ; then
	//	echo ${ret}
	//	return 0
	//fi

	//#
	//# Then we try /etc/env.d/gcc/config-${CTARGET}
	//#
	//if [[ -s ${EROOT}/etc/env.d/gcc/config-${CTARGET} ]] ; then
	//	ret=$(split_gcc_ver $(show_var CURRENT "${EROOT}"/etc/env.d/gcc/config-${CTARGET}))
	//	echo ${ret% *}
	//fi
		}
	} //unnamed namespace

	std::string chost(const SysPaths& sp) {
		using std::string;

		{
			// If it's set in the env, trust the setting.  If it's wrong,
			// then that's the caller's problem.
			auto ret = sp.env("CHOST");
			if (ret and not ret->empty())
				return string(*ret);
		}

		const int python_ret = run_cmd_retcode("python -V &>/dev/null");
		std::string chost_str;

		if (not python_ret) {
			chost_str = run_cmd("portageq", "envvar", "CHOST").out;
		}
		else {
			ewarn(sp, "Python seems to be broken, attempting to locate CHOST ourselves ...");
			chost_str = try_real_hard_to_find_CHOST();
		}

		if (chost_str.empty()) {
			eerror(sp, replace_bash_vars(sp, "${argv0}: Could not get portage CHOST!"));
			eerror(sp, replace_bash_vars(sp, "${argv0}: You should verify that CHOST is set in one of these places:"));
			eerror(sp, replace_bash_vars(sp, "${argv0}:  - ${EROOT}/etc/portage/make.conf"));
			eerror(sp, replace_bash_vars(sp, "${argv0}:  - active environment"));
			throw std::runtime_error(replace_bash_vars(sp, "${argv0}: Could not get portage CHOST!"));
		}

		return chost_str;
	}
} //namespace duck
