/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <string_view>
#include <optional>
#include <utility>

namespace duck {
	class EnvBase {
	public:
		typedef std::string_view mapped_type;
		typedef std::string_view key_type;
		typedef std::pair<const key_type, mapped_type> value_type;
		typedef std::size_t size_type;
		typedef std::ptrdiff_t difference_type;

		EnvBase() = default;
		virtual ~EnvBase() noexcept = default;

		virtual std::string_view operator[] (std::string_view name) const noexcept = 0;
		virtual std::optional<std::string_view> get (std::string_view name) const noexcept = 0;
		virtual std::string_view get (std::string_view name, std::string_view def) const noexcept = 0;
		virtual void set (std::string_view name, std::string_view value) = 0;
		virtual void unset (std::string_view name) noexcept = 0;
		virtual bool is_set (std::string_view name) const noexcept = 0;
		virtual size_type size() const noexcept = 0;
		virtual bool empty() const noexcept;
		virtual void clear() noexcept = 0;
	};
} //namespace duck
