/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "env_base.hpp"
#include <memory>

namespace duck {
	class EnvReal : public EnvBase {
	public:
		EnvReal();
		virtual ~EnvReal() noexcept = default;

		virtual std::string_view operator[] (std::string_view name) const noexcept override;
		virtual std::optional<std::string_view> get (std::string_view name) const noexcept override;
		virtual std::string_view get (std::string_view name, std::string_view def) const noexcept override;
		virtual void set (std::string_view name, std::string_view value) override;
		virtual void unset (std::string_view name) noexcept override;
		virtual bool is_set (std::string_view name) const noexcept override;
		virtual size_type size() const noexcept override;
		virtual void clear() noexcept override;

	private:
		const char* raw_fetch_env (std::string_view name) const noexcept;

		std::unique_ptr<char[]> m_name_buff;
		std::size_t m_name_buff_len;
	};
} //namespace duck
