/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "replace_bash_vars.hpp"
#include "config.h"
#include "syspaths.hpp"
#include <cassert>
#include <cctype>

namespace duck {
	namespace {
		struct VarInfo {
			std::string_view name;
			std::size_t orig_len{};
		};

		//std::string replaced_all (std::string text, std::string_view search, std::string_view repl) {
		//	std::size_t pos = 0;
		//	while ((pos = text.find(search, pos)) != std::string::npos) {
		//		text.replace(pos, search.size(), repl);
		//		pos += repl.size();
		//	}
		//	return text;
		//}

		VarInfo extract_var_name (const SysPaths& sp, std::string_view text) {
			assert(not text.empty());
			assert(text.front() == '$');
			if (text.size() == 1) {
				return {};
			}
			else if (text.size() == 2) {
				if (std::isalnum(text[1]))
					return {text.substr(1, 1), 2};
				return {};
			}
			else {
				assert(text.size() >= 3);
				if (text[1] == '{') {
					const auto end = text.find('}', 2);
					if (text.npos == end) {
						return {};
					}
					else {
						std::string_view token(text.substr(2, end - 2));
						std::string_view retval(token.empty() || token.front() != '!' ? token : token.substr(1));
						if (not token.empty() and token.front() == '!') {
							return {sp[retval], end + 1};
						}
						else {
							return {retval, end + 1};
						}
					}
				}
				else {
					std::size_t z;
					for (z = 1; z < text.size() && std::isalnum(text[z]); ++z) {
					}
					return {text.substr(1, z - 1), z + 1};
				}
			}
		}
	} //unnamed namespace

	std::string replace_bash_vars (const SysPaths& sp, std::string_view text) {
		std::size_t pos = 0;
		std::size_t prev_start = 0;
		std::string rtext;
		rtext.reserve(text.size());

		while ((pos = text.find('$', pos)) != std::string::npos) {
			if (0 == pos or rtext[pos-1] != '\\') {
				auto var = extract_var_name(sp, text.substr(pos));
				if (not var.orig_len) {
					++pos;
					continue;
				}

				assert(prev_start <= pos);
				rtext.append(text.begin() + prev_start, text.begin() + pos);
				if (var.name == "argv0")
					rtext += PROJECT_NAME;
				else
					rtext += sp[var.name];
				pos += var.orig_len;
				prev_start = pos;
			}
			else {
				++pos;
			}
		}

		rtext.append(text.begin() + prev_start, text.end());
		return rtext;
	}
} //namespace duck
