/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "string_view_cat.hpp"
#include <string>
#include <string_view>
#include <utility>
#include <vector>

namespace duck {
	struct RunCmdOutput {
		std::string out;
		std::string err;
	};

	namespace detail {
		RunCmdOutput run_cmd(const std::string& cmd, const std::vector<std::string>& argv);
	} //namespace detail

	template <typename... Args>
	inline RunCmdOutput run_cmd (std::string command, Args&&... args) {
		using duck::to_string;

		std::vector<std::string> argv {
			command,
			to_string(std::forward<Args>(args))...
		};
		return detail::run_cmd(command, argv);
	}

	int run_cmd_retcode (const char* command);
} //namespace duck
