/* Copyright 2020, Michele Santullo
 * This file is part of user-gcc.
 *
 * User-gcc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * User-gcc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with user-gcc.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "var_map.hpp"
#include <filesystem>
#include <string>
#if !defined(NDEBUG)
#	include <iostream>
#endif
#include <unordered_map>
#include <string_view>
#include <optional>

namespace duck {
	class SysPaths {
	public:
		typedef std::filesystem::path path_type;

		SysPaths();
		~SysPaths() noexcept = default;

		std::string_view operator[] (std::string_view name) const noexcept;
		std::optional<std::string_view> env (std::string_view name) const noexcept;
		std::string_view env (std::string_view name, std::string_view def) const noexcept;

		void set_colours (std::string_view colours);
		VarMapView colours() const { return {&m_colours}; }

		const path_type& root() const;
		const path_type& eroot() const;
		const path_type& eprefix() const;
		const path_type& env_d() const;
		const path_type& gcc_env_d() const;

	private:
		VarMap m_env;
		VarMap m_colours;
		path_type m_root;
		path_type m_eprefix;
		path_type m_eroot;
		path_type m_env_d;
		path_type m_gcc_env_d;
	};

#if !defined(NDEBUG)
	std::ostream& operator<< (std::ostream&, const SysPaths& sp);
#endif
} //namespace duck
